package me.icymint.libra.sage.model;

import java.sql.SQLException;
import java.util.Collection;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Queue;
import java.util.TreeMap;

import me.icymint.libra.sage.model.format.SqlFormatException;
import me.icymint.libra.sage.model.object.ProcedureManagerObject;
import me.icymint.libra.sage.model.object.SqlObject;
import me.icymint.libra.sage.model.object.SqlObjectType;
import me.icymint.libra.sage.model.operator.ClassNotMatchException;
import me.icymint.libra.sage.model.operator.InjectException;
import me.icymint.libra.sage.model.operator.Injector;
import me.icymint.libra.sage.model.operator.ManagerArray;
import me.icymint.libra.sage.model.operator.ManagerObjects;
import me.icymint.libra.sage.model.operator.ManagerTypes;
import me.icymint.libra.sage.model.operator.SQLExistsException;
import me.icymint.libra.sage.model.operator.SQLNotExistsException;
import me.icymint.libra.sage.model.token.ProcedureToken;

/**
 * SQL数据库对象。
 * 
 * @author Daniel
 * @datetime 2012-11-14上午9:39:39
 * 
 */
public class SqlDatabase extends ProcedureManagerObject implements Injector {
	private ManagerArray<SqlInjector, Class<?>> ma;
	private Queue<ProcedureToken> precall = null;
	private ManagerObjects<SqlType> mi;

	protected SqlDatabase(SqlInfo si, String prefix, String name,
			boolean isMask, SqlTable[] st, SqlFunction[] sf, SqlProcedure[] sp) {
		super(si, prefix, name, isMask, null, st, sf, sp);
		ma = new ManagerArray<SqlInjector, Class<?>>(null, SqlInjector.class) {

			@Override
			protected Class<?> getName(SqlInjector t) {
				return t.getSupportClass();
			}

			@Override
			protected boolean isEqual(Class<?> v1, Class<?> v2) {
				return v1.equals(v2);
			}

		};
		mi = new ManagerTypes();
	}

	public void addInjector(SqlInjector si) throws SQLExistsException {
		ma.add(si);
	}

	public void addType(SqlType t) {
		try {
			mi.add(t);
		} catch (SQLExistsException e) {
		}
	}

	/**
	 * 在数据库中新建对应的实体。
	 * 
	 * @throws SQLException
	 */
	public void build() throws SQLException {
		long l = System.currentTimeMillis();
		try {
			long ll = System.currentTimeMillis();

			for (String abc : crt(this.getTables())) {
				getJdbcUtils().execute(abc);
			}
			logger.debug("创建表格完毕！耗时" + (System.currentTimeMillis() - ll) + "ms");
			ll = System.currentTimeMillis();
			for (String abc : crf(this.getFunctions())) {
				getJdbcUtils().execute(abc);
			}
			logger.debug("创建自定义函数完毕！耗时" + (System.currentTimeMillis() - ll)
					+ "ms");
			ll = System.currentTimeMillis();
			for (String abc : crp(this.getProcedures())) {
				getJdbcUtils().execute(abc);
			}
			logger.debug("创建存储过程完毕！耗时" + (System.currentTimeMillis() - ll)
					+ "ms");
			ll = System.currentTimeMillis();
			if (precall != null)
				for (ProcedureToken pt : precall) {
					pt.commit();
				}
			logger.debug("初始化数据导入完毕！耗时" + (System.currentTimeMillis() - ll)
					+ "ms");
		} catch (SQLException e1) {
			throw new SQLException("数据库创建失败！", e1);
		}
		logger.debug("创建数据库完毕！耗时" + (System.currentTimeMillis() - l) + "ms");
	}

	/**
	 * 清除所有数据库中的实体。
	 * 
	 * @throws SQLException
	 */
	public void clean() throws SQLException {
		long l = System.currentTimeMillis();
		this.getJdbcUtils().executeBatchs(
				sqlp(select(this.getJdbcUtils().getProcedures(),
						this.getProcedures(), SqlProcedure.class).getAll()));
		this.getJdbcUtils().executeBatchs(
				sqlf(select(this.getJdbcUtils().getFunctions(),
						this.getFunctions(), SqlFunction.class).getAll()));
		this.getJdbcUtils().executeBatchs(
				sqlt(select(this.getJdbcUtils().getTables(), this.getTables(),
						SqlTable.class).getAll()));
		logger.debug("删除数据库完毕！耗时" + (System.currentTimeMillis() - l) + "ms");
	}

	private String[] crf(SqlFunction[] all) throws SqlFormatException {
		String[] sql = new String[all.length];
		for (int i = 0; i < all.length; i++) {
			sql[i] = this.getDialect().getCreateFunctionSql(all[i]);
		}
		return sql;
	}

	private String[] crp(SqlProcedure[] all) throws SqlFormatException {
		String[] sql = new String[all.length];
		for (int i = 0; i < all.length; i++) {
			sql[i] = this.getDialect().getCreateProcedureSql(all[i]);
		}
		return sql;
	}

	private String[] crt(SqlTable[] all) throws SqlFormatException {
		Map<Integer, Collection<String>> map = new TreeMap<Integer, Collection<String>>();
		for (int i = 0; i < all.length; i++) {
			int level = -all[i].level();
			Collection<String> coll = map.get(level);
			if (coll == null) {
				coll = new LinkedList<String>();
				map.put(level, coll);
			}
			coll.add(this.getDialect().getCreateTableSql(all[i]));
		}
		List<String> coll = new LinkedList<String>();
		for (int i : map.keySet()) {
			coll.addAll(map.get(i));
		}
		return coll.toArray(new String[] {});
	}

	public Queue<ProcedureToken> getInitialData() {
		return precall;
	}

	public SqlInjector[] getInjectors() {
		return ma.getAll();
	}

	@Override
	protected SqlObjectType getSqlObjectType() {
		return SqlObjectType.Database;
	}

	public SqlType getType(int index) throws SQLNotExistsException {
		return mi.get(index);
	}

	public SqlType getType(String name) throws SQLNotExistsException {
		return mi.get(name);
	}

	public SqlType[] getTypes() {
		return mi.getAll();
	}

	@Override
	protected String getVarId(SqlVar v) {
		return v.getId();
	}

	@Override
	public void inject(Object obj) throws ClassNotMatchException,
			InjectException {
		Class<?> cls = obj.getClass();
		SqlInjector si = null;
		do {
			try {
				si = ma.get(cls);
			} catch (SQLNotExistsException e) {
				cls = cls.getSuperclass();
				if (cls == null) {
					logger.error("对象" + obj + "没有对应的适配注入器。");
					throw new ClassNotMatchException();
				}
			}
		} while (si == null);
		si.inject(obj);
	}

	/**
	 * 重建数据库。
	 * 
	 * @throws SQLException
	 * 
	 */
	public void rebuild() throws SQLException {
		try {
			long l = System.currentTimeMillis();
			clean();
			build();
			logger.debug("重建数据库完毕！耗时" + (System.currentTimeMillis() - l) + "ms");
			logger.debug("SQL目前执行次数" + this.getJdbcUtils().get());
		} catch (SQLException e) {
			throw new SQLException("数据库重建失败！", e);
		}
	}

	private <T extends SqlObject> ManagerObjects<T> select(List<String> names,
			T[] origin, Class<T> cls) {
		ManagerObjects<T> mo = new ManagerObjects<T>(origin, cls);
		for (T t : origin) {
			if (!names.contains(t.toString().toLowerCase())) {
				mo.del(t);
			}
		}
		return mo;
	}

	public void setInitialData(Queue<ProcedureToken> precall) {
		this.precall = precall;
	}

	private String[] sqlf(SqlFunction[] all) throws SqlFormatException {
		String[] sql = new String[all.length];
		for (int i = 0; i < all.length; i++) {
			sql[i] = this.getDialect().getDeleteFunctionSql(all[i]);
		}
		return sql;
	}

	private String[] sqlp(SqlProcedure[] all) throws SqlFormatException {
		String[] sql = new String[all.length];
		for (int i = 0; i < all.length; i++) {
			sql[i] = this.getDialect().getDeleteProcedureSql(all[i]);
		}
		return sql;
	}

	private String[] sqlt(SqlTable[] all) throws SqlFormatException {
		Map<Integer, Collection<String>> map = new TreeMap<Integer, Collection<String>>();
		for (int i = 0; i < all.length; i++) {
			int level = all[i].level();
			Collection<String> coll = map.get(level);
			if (coll == null) {
				coll = new LinkedList<String>();
				map.put(level, coll);
			}
			coll.add(this.getDialect().getDeleteTableSql(all[i]));
		}
		List<String> coll = new LinkedList<String>();
		for (int i : map.keySet()) {
			coll.addAll(map.get(i));
		}
		return coll.toArray(new String[] {});
	}
}
