package me.icymint.libra.sage.model;

import me.icymint.libra.sage.model.object.SqlObject;

/**
 * 该类封装了java.sql.Types里面的基础信息，以及长度和默认值。
 * 
 * @author Daniel
 * @datetime 2012-11-12下午5:26:46
 * 
 */
public class SqlType implements SqlObject {

	private final int length;

	private final String defaultvalue;

	private final SqlVarType type;

	private final int id;

	private final SqlInfo si;

	protected SqlType(SqlInfo si, SqlVarType type, int len, String def, int id) {
		this.si = si;
		this.length = len;
		this.defaultvalue = def;
		if (type == null)
			throw new NullPointerException("SqlType必须指定。");
		this.type = type;
		this.id = id;
	}

	public boolean equals(SqlType t) {
		return this.key().equals(t.key());
	}

	public String getDefaultValue() {
		return this.defaultvalue;
	}

	@Override
	public String getId() {
		return toString() + "," + defaultvalue;
	}

	public int getIndex() {
		return id;
	}

	public String getJdbcName() {
		return type.name();
	}

	public int getJdbcType() {
		return type.getIndex();
	}

	public int getLength() {
		return length;
	}

	@Override
	public String getName() {
		return si.getJdbcUtils().getDialect().transfer(type);
	}

	@Deprecated
	@Override
	public String getPrefix() {
		return "";
	}

	@Deprecated
	@Override
	public SqlInfo getSqlInfo() {
		return si;
	}

	public boolean hasLength() {
		return type.hasLength() ? length > 0 : false;
	}

	@Deprecated
	@Override
	public boolean isMask() {
		return false;
	}

	private String key() {
		return this.toString() + "|" + this.getDefaultValue();
	}

	@Override
	public String toString() {
		if (type.hasLength() && length > 0)
			return getName() + "(" + length + ")";
		else
			return getName();
	}
}
