package me.icymint.libra.sage.model.format;

import java.util.regex.Matcher;

/**
 * 特定ManagerObjects格式化。
 * 
 * @author Daniel
 * 
 */
public abstract class AbstractFormat implements FormatSql {
	protected static String NAME = "\\s*([_a-zA-Z][_a-zA-Z0-9])*\\s*";
	protected static String ID = "\\s*(\\d+)\\s*";
	protected static String NAME_ID = "\\s*([_a-zA-Z][_a-zA-Z0-9]*|\\d+)\\s*";

	private final PatternFormat[] pf;

	public AbstractFormat(PatternFormat... pf) {
		this.pf = pf;
	}

	/**
	 * 格式化字符串，这里定义了{@link PatternFormat}需要完成的任务。
	 */
	@Override
	public String format(String sql) throws SqlFormatException {
		for (PatternFormat p : this.pf) {
			Matcher m = p.getPattern().matcher(sql);
			String temp = new String(sql);
			while (m.find()) {
				temp = temp.replace(m.group(), p.format(temp, m));
			}
			sql = temp;
		}
		sql = sql.trim();
		if (!sql.endsWith(";"))
			sql += ";";
		return sql;
	}

}
