package me.icymint.libra.sage.model.format;

import me.icymint.libra.sage.model.SqlFunction;
import me.icymint.libra.sage.model.SqlTable;
import me.icymint.libra.sage.model.SqlVar;

/**
 * Sql对象内部Sql代码预处理接口。 包括输入输出的格式化，以及解析。
 * <p>
 * 一般的，特定Sql代码需要对应特定的Sql对象，该对象内部会容纳其它Sql对象。 {@link SqlFunction}对象容纳
 * {@link SqlTable}对象和{@link SqlVar}对象，分别表示{@link SqlFunction}需要调用的
 * {@link SqlTable}和其变量值。 同时，{@link SqlFunction}对象存在一条原始的函数体代码Sql。 <br/>
 * 该Sql语句如下所示： <blockquote>
 * 
 * <pre>
 * RETURN MD5($v(str));
 * </pre>
 * 
 * </blockquote> 经过格式化以后，其会被替换成： <blockquote> Mysql:
 * 
 * <pre>
 * RETURN MD5(str);
 * </pre>
 * 
 * SqlServer:
 * 
 * <pre>
 * RETURN MD5(@str);
 * </pre>
 * 
 * </blockquote> 两者的解析结果是不同的，根据各自的需要调整。 实现该接口的类需要完成一项或多项此类转换。
 * 
 * @author Daniel
 * 
 */
public interface FormatSql {
	/**
	 * 基础格式化类，紧凑化。
	 */
	FormatSql COMPACT = new CompactFormat();

	/**
	 * 按照特定规则格式化成新的字符串。
	 * 
	 * @param sql
	 * @throws SqlFormatException
	 *             当格式化失败的时候，抛出此违例。
	 */
	String format(String sql) throws SqlFormatException;
}
