package me.icymint.libra.sage.model.format;

import java.util.LinkedList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import me.icymint.libra.sage.model.SqlTable;
import me.icymint.libra.sage.model.operator.FunctionManager;
import me.icymint.libra.sage.model.operator.SQLNotExistsException;
import me.icymint.libra.sage.model.operator.TableManager;
import me.icymint.libra.sage.model.operator.VarManager;

/**
 * 把Sql代码中的名称指标转换成数字指标。 例如：$t(table1)表示第1张表table1，经过格式化以后，其变成$t(0)。
 * 注意，数字指标从0开始计数。
 * 
 * @author Daniel
 * @datetime 2012-11-14上午9:10:26
 * 
 */
public class NameToNumberFormat extends AbstractFormat {
	private static PatternFormat[] get(Object obj) {
		List<PatternFormat> pf = new LinkedList<PatternFormat>();
		if (obj instanceof VarManager) {
			pf.add(getVarFormat((VarManager) obj));
		}
		if (obj instanceof TableManager) {
			pf.add(getTableFormat((TableManager) obj));
			pf.add(getTableVarFormat((TableManager) obj));
		}
		if (obj instanceof FunctionManager) {
			pf.add(getFunctionFormat((FunctionManager) obj));
		}
		return pf.toArray(new PatternFormat[] {});

	}

	public static PatternFormat getFunctionFormat(final FunctionManager fm) {
		return new PatternFormat(Pattern.compile("\\$f\\(" + NAME + "\\)")) {

			@Override
			public String format(String sql, Matcher m)
					throws SqlFormatException {
				try {
					return "$f(" + fm.getFunctionIndex(m.group(1)) + ")";
				} catch (SQLNotExistsException e) {
					throw new SqlFormatException(errorMessage(sql, m.group(),
							"函数"), e);
				}
			}

		};
	}

	public static PatternFormat getTableFormat(final TableManager tm) {
		return new PatternFormat(Pattern.compile("\\$t\\(" + NAME + "\\)")) {

			@Override
			public String format(String sql, Matcher m)
					throws SqlFormatException {
				try {
					return "$t(" + tm.getTableIndex(m.group(1)) + ")";
				} catch (SQLNotExistsException e) {
					throw new SqlFormatException(errorMessage(sql, m.group(),
							"表格"), e);
				}
			}

		};
	}

	public static PatternFormat getTableVarFormat(final TableManager tm) {
		return new PatternFormat(Pattern.compile("\\$t\\(" + NAME + "," + NAME
				+ "\\)")) {

			@Override
			public String format(String sql, Matcher m)
					throws SqlFormatException {

				SqlTable t = null;
				int tind;
				try {
					t = tm.getTable(m.group(1));
					tind = tm.getTableIndex(m.group(1));
				} catch (SQLNotExistsException e) {
					throw new SqlFormatException(errorMessage(sql, m.group(),
							"表格"), e);
				}

				try {
					return "$t(" + tind + "," + t.getVarIndex(m.group(2)) + ")";
				} catch (SQLNotExistsException e) {
					throw new SqlFormatException(errorMessage(sql, m.group(),
							"字段"), e);
				}
			}

		};
	}

	public static PatternFormat getVarFormat(final VarManager vm) {
		return new PatternFormat(Pattern.compile("\\$v\\(" + NAME + "\\)")) {

			@Override
			public String format(String sql, Matcher m)
					throws SqlFormatException {
				try {
					return "$v(" + vm.getVarIndex(m.group(1)) + ")";
				} catch (SQLNotExistsException e) {
					throw new SqlFormatException(errorMessage(sql, m.group(),
							"字段"), e);
				}
			}

		};
	}

	public NameToNumberFormat(Object obj) {
		this(get(obj));
	}

	public NameToNumberFormat(PatternFormat... pf) {
		super(pf);
	}
}
