package me.icymint.libra.sage.model.orm;

import java.sql.SQLException;
import java.util.Collection;
import java.util.List;

import me.icymint.libra.sage.core.JdbcUtils;
import me.icymint.libra.sage.core.dialect.SqlDialect;
import me.icymint.libra.sage.model.SqlTable;
import me.icymint.libra.sage.model.SqlVar;
import me.icymint.libra.sage.model.operator.Curd;

/**
 * 表格的增删查改。
 * 
 * @author Daniel
 * @datetime 2012-12-12下午2:19:33
 * 
 */
public class TableCurd implements Curd {
	private final SqlTable st;
	private final JdbcUtils ju;
	private final SqlDialect sd;

	public TableCurd(SqlTable t) {
		st = t;
		ju = st.getJdbcUtils();
		sd = st.getDialect();
	}

	@Override
	public <T> void delete(T t) throws SQLException {
		ju.update(sd.getDeleteByNames(st, st.getVars()), st.getVars(), t);
	}

	@Override
	public <T> void deleteByKey(T t) throws SQLException {
		ju.update(sd.getDeleteByNames(st, st.getPrimaryKeys()),
				st.getPrimaryKeys(), t);
	}

	protected SqlVar[] filter(SqlVar[] vs) throws SQLException {
		if (st.isAutoIncrement()) {

			try {
				SqlVar[] vvs = new SqlVar[vs.length
						- st.getPrimaryKeys().length];
				int i = 0;
				for (SqlVar v : vs) {
					if (!st.isKey(v.getName())) {
						vvs[i++] = v;
					}
				}
				vs = vvs;
			} catch (ArrayIndexOutOfBoundsException e) {
				throw new SQLException("表格" + st.getName() + "可能不正常", e);
			}
		}
		return vs;
	}

	protected SqlVar[] filterKey(SqlVar[] vs) throws SQLException {
		try {
			SqlVar[] vvs = new SqlVar[vs.length - st.getPrimaryKeys().length];
			int i = 0;
			for (SqlVar v : vs) {
				if (!st.isKey(v.getName())) {
					vvs[i++] = v;
				}
			}
			vs = vvs;
		} catch (ArrayIndexOutOfBoundsException e) {
			throw new SQLException("表格" + st.getName() + "可能不正常", e);
		}
		return vs;
	}

	@Override
	public <T> void insert(T t) throws SQLException {
		ju.update(sd.getInsert(st), filter(st.getVars()), t);
	}

	@Override
	public <T> void insertAll(Collection<T> coll) throws SQLException {
		ju.updateBatch(sd.getInsert(st), filter(st.getVars()), coll);
	}

	@Override
	public List<DataPack> query() throws SQLException {
		return query(DataPack.class);
	}

	@Override
	public <T> List<T> query(Class<T> cls) throws SQLException {
		return this.query(cls, 1, 0);
	}

	@Override
	public <T> List<T> query(Class<T> cls, int offset, int limit)
			throws SQLException {
		return ju.loadList(sd.getQueryAll(st), cls, st.getVars(), offset,
				limit, null);
	}

	@Override
	public List<DataPack> query(int offset, int limit) throws SQLException {
		return this.query(DataPack.class, offset, limit);
	}

	@Override
	public <T> T queryByKey(Class<T> cls, Object... key) throws SQLException {
		return ju.load(sd.getQueryByNames(st, st.getPrimaryKeys()), cls,
				st.getVars(), st.getPrimaryKeys(), key);
	}

	@Override
	public DataPack queryByKey(Object... key) throws SQLException {
		return queryByKey(DataPack.class, key);
	}

	@Override
	public <T> T sync(T t) throws SQLException {
		if (t != null && st.isAutoIncrement()) {
			SqlVar[] xx = filterKey(st.getVars());
			try {
				DataPack dp = ju.load(sd.getQueryByNames(st, xx), st.getVars(),
						xx, ju.getMappings().unpack(t, xx));
				SqlVar v = st.getPrimaryKeys()[0];
				st.setValueOf(t, v, dp.getValue(v));
			} catch (PackException e) {
				throw new SQLException(e);
			}
		}
		return t;
	}

	@Override
	public <T> void update(T t) throws SQLException {
		ju.update(
				sd.getUpdate(st, filterKey(st.getVars()), st.getPrimaryKeys()),
				st.getPrimaryKeys(), t);

	}
}
