package me.icymint.libra.sage.model.xml;

import java.io.IOException;
import java.io.InputStream;
import java.io.UnsupportedEncodingException;
import java.net.URL;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import me.icymint.libra.sage.model.SqlFactory;
import me.icymint.libra.sage.model.SqlInfo;

import org.apache.log4j.Logger;
import org.dom4j.Document;
import org.dom4j.DocumentException;
import org.dom4j.Element;
import org.dom4j.io.OutputFormat;
import org.dom4j.io.SAXReader;
import org.dom4j.io.SAXValidator;
import org.dom4j.io.XMLWriter;
import org.dom4j.util.XMLErrorHandler;
import org.xml.sax.SAXException;

/**
 * 抽象解析类。 该类提供了验证xml流的方案以及其它一些半成品。
 * 
 * @author Daniel
 * @datetime 2012-11-2下午1:48:50
 * 
 */
public abstract class AbstractSqlParser implements SqlParser {
	protected Logger logger = Logger.getLogger(getClass());

	private Element root = null;

	private final URL xsd;

	protected AbstractSqlParser(URL xsd) {
		if (xsd == null)
			throw new NullPointerException();
		this.xsd = xsd;
	}

	protected abstract SqlFactory parserXml(Element root, SqlInfo si)
			throws XmlParserException;

	@Override
	public SqlFactory parserXml(InputStream xml, SqlInfo si)
			throws XmlParserException {
		if (!this.validateXml(xml))
			throw new XmlParserException();
		return parserXml(root, si);
	}

	@Override
	public boolean validateXml(InputStream xml) {
		XMLErrorHandler errorHandler = new XMLErrorHandler();
		SAXParserFactory factory = SAXParserFactory.newInstance();
		factory.setValidating(true);
		factory.setNamespaceAware(true);
		try {
			SAXParser parser = factory.newSAXParser();
			SAXReader xmlReader = new SAXReader();
			xmlReader.setEncoding("UTF-8");
			Document xmlDocument = xmlReader.read(xml);
			parser.setProperty(
					"http://java.sun.com/xml/jaxp/properties/schemaLanguage",
					"http://www.w3.org/2001/XMLSchema");
			parser.setProperty(
					"http://java.sun.com/xml/jaxp/properties/schemaSource",
					xsd.openStream());
			SAXValidator validator = new SAXValidator(parser.getXMLReader());
			validator.setErrorHandler(errorHandler);
			validator.validate(xmlDocument);
			XMLWriter writer = new XMLWriter(OutputFormat.createPrettyPrint());

			if (errorHandler.getErrors().hasContent()) {
				logger.error("XML文件通过XSD文件校验失败！");
				writer.write(errorHandler.getErrors());
			} else {
				root = xmlDocument.getRootElement();
				return true;
			}
		} catch (ParserConfigurationException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (SAXException e) {
			logger.error("解析失败！");
			e.printStackTrace();
		} catch (DocumentException e) {
			logger.error("文档输入流错误！");
			e.printStackTrace();
		} catch (UnsupportedEncodingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return false;
	}

}
