package me.icymint.libra.sage.model.xml;

import java.beans.PropertyEditor;
import java.io.IOException;
import java.io.OutputStream;
import java.io.UnsupportedEncodingException;
import java.sql.SQLException;
import java.util.Arrays;
import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.LinkedHashSet;
import java.util.LinkedList;
import java.util.Map;
import java.util.Queue;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import me.icymint.libra.sage.core.dialect.Dialect;
import me.icymint.libra.sage.core.dialect.SqlDialect;
import me.icymint.libra.sage.model.SqlDatabase;
import me.icymint.libra.sage.model.SqlFactory;
import me.icymint.libra.sage.model.SqlFunction;
import me.icymint.libra.sage.model.SqlInfo;
import me.icymint.libra.sage.model.SqlInjector;
import me.icymint.libra.sage.model.SqlProcedure;
import me.icymint.libra.sage.model.SqlTable;
import me.icymint.libra.sage.model.SqlType;
import me.icymint.libra.sage.model.SqlVar;
import me.icymint.libra.sage.model.SqlVarType;
import me.icymint.libra.sage.model.format.FormatSql;
import me.icymint.libra.sage.model.format.IndentFormat;
import me.icymint.libra.sage.model.format.NameToNumberFormat;
import me.icymint.libra.sage.model.format.NumberToNameFormat;
import me.icymint.libra.sage.model.format.SqlFormatException;
import me.icymint.libra.sage.model.object.AbstractSql;
import me.icymint.libra.sage.model.object.SqlObject;
import me.icymint.libra.sage.model.operator.FunctionManager;
import me.icymint.libra.sage.model.operator.ManagerArray;
import me.icymint.libra.sage.model.operator.ManagerObjects;
import me.icymint.libra.sage.model.operator.ManagerTypes;
import me.icymint.libra.sage.model.operator.ProcedureManager;
import me.icymint.libra.sage.model.operator.SQLExistsException;
import me.icymint.libra.sage.model.operator.SQLNotExistsException;
import me.icymint.libra.sage.model.operator.TableManager;
import me.icymint.libra.sage.model.operator.VarManager;
import me.icymint.libra.sage.model.orm.Mapping;
import me.icymint.libra.sage.model.orm.MappingException;
import me.icymint.libra.sage.model.orm.PackUtils;
import me.icymint.libra.sage.model.token.ParamNotMatchException;
import me.icymint.libra.sage.model.token.ProcedureToken;

import org.apache.log4j.Logger;
import org.dom4j.Document;
import org.dom4j.DocumentFactory;
import org.dom4j.DocumentHelper;
import org.dom4j.Element;
import org.dom4j.Namespace;
import org.dom4j.io.OutputFormat;
import org.dom4j.io.XMLWriter;

/**
 * 默认的解析版本。
 * 
 * @author Daniel
 * @datetime 2012-11-2下午1:48:23
 * 
 */
class DefaultSqlParser extends AbstractSqlParser {

	private static class XmlReader {
		protected Logger logger = Logger.getLogger(getClass());

		private static Pattern p = Pattern
				.compile("([a-zA-Z][0-9a-zA-Z_]+)\\((\\d+)(,'(.*?)')?\\)");
		private static Pattern v = Pattern
				.compile("([a-zA-Z][0-9a-zA-Z_]+|\\d+)");
		private static Pattern t = Pattern.compile("([a-zA-Z][0-9a-zA-Z_]+)");
		private static Pattern c = Pattern
				.compile("#([a-zA-Z][0-9a-zA-Z_]{0,47})\\{\\s*(('[^{}]+'(\\s*,\\s*'[^{}]+')*)\\s*;\\s*)+\\}");
		private static Pattern cv = Pattern.compile("\\s*(.+?)\\s*;");
		private static Pattern cp = Pattern.compile("'(.*?)'");
		private static Pattern pv = Pattern
				.compile("([a-zA-Z][0-9a-zA-Z_]*)=([a-zA-Z][0-9a-zA-Z_]*)\\.([a-zA-Z][0-9a-zA-Z_]*)");

		private String get(Element e, String sub, String attr) {
			Element ee = e.element(sub);
			if (ee == null)
				return "";
			else {
				attr = ee.attributeValue(attr);
				return attr == null ? "" : attr;
			}
		}

		private SqlFunction[] getFunctions(Element e, String attr, SqlFactory sf)
				throws XmlParserException {
			String value = e.attributeValue(attr);
			ManagerObjects<SqlFunction> mo = new ManagerObjects<SqlFunction>(
					SqlFunction.class);
			if (value != null) {
				Matcher m = t.matcher(value);
				while (m.find()) {
					try {
						mo.add(sf.getDatabase().getFunction(m.group(1)));
					} catch (SQLExistsException e1) {
						logger.warn("函数" + m.group(1) + "重复！该函数不再被添加。");
					} catch (SQLNotExistsException e1) {
						throw new XmlParserException("函数" + m.group(1) + "不存在！"
								+ e.getName() + "-" + e.attributeValue("name")
								+ "解析失败！");
					}
				}
			}
			return mo.getAll();
		}

		private SqlTable[] getTables(Element e, String attr, SqlFactory sf)
				throws XmlParserException {
			String value = e.attributeValue(attr);
			ManagerObjects<SqlTable> mo = new ManagerObjects<SqlTable>(
					SqlTable.class);
			if (value != null) {
				Matcher m = t.matcher(value);
				while (m.find()) {
					try {
						mo.add(sf.getDatabase().getTable(m.group(1)));
					} catch (SQLNotExistsException e1) {
						throw new XmlParserException("表格" + m.group(1) + "不存在！"
								+ e.getName() + "-" + e.attributeValue("name")
								+ "解析失败！", e1);
					} catch (SQLExistsException e1) {
						logger.warn("表格" + m.group(1) + "重复！该表格不再被添加。");
					}
				}
			}
			return mo.getAll();
		}

		private SqlVar[] getVars(Element e, String attr, SqlFactory sf)
				throws XmlParserException {
			String vs = e.attributeValue(attr);
			ManagerObjects<SqlVar> mo = new ManagerObjects<SqlVar>(SqlVar.class);
			if (vs != null) {
				Matcher m = p.matcher(vs);
				while (m.find()) {
					try {
						mo.add(sf.createVar(m.group(1), sf.getDatabase()
								.getType(Integer.valueOf(m.group(2))), m
								.group(4)));
					} catch (SQLNotExistsException e1) {
						throw new XmlParserException("类型序号" + m.group(2)
								+ "不存在！" + e.getName() + "-"
								+ e.attributeValue("name") + "解析失败！", e1);
					} catch (SQLExistsException e1) {
						logger.warn("字段" + m.group(1) + "重复！该字段不再被添加。");
					}
				}
			}
			return mo.getAll();
		}

		private Queue<ProcedureToken> parserCall(Element e, SqlFactory sf)
				throws XmlParserException {
			Queue<ProcedureToken> calls = new LinkedList<ProcedureToken>();
			Matcher m = c.matcher(e.getTextTrim());
			while (m.find()) {
				ProcedureToken spt;
				try {
					spt = new ProcedureToken(sf.getDatabase().getProcedure(
							m.group(1)));
				} catch (SQLNotExistsException e1) {
					throw new XmlParserException("未找到存储过程" + m.group(1)
							+ ",call解析失败", e1);
				}
				/**
				 * 解析值列表。
				 */
				Matcher mv = cv.matcher(m.group(2));
				while (mv.find()) {
					/**
					 * 解析单条记录。
					 */
					Matcher mp = cp.matcher(mv.group(1));
					Object[] sv = new Object[spt.getParamSize()];
					int i = 0;
					while (mp.find()) {
						sv[i] = mp.group(1);
						i++;
					}
					// System.out.println(sv.length+"--"+Arrays.toString(sv));
					try {
						spt.addTokens(sv);
					} catch (ParamNotMatchException e1) {
						throw new XmlParserException("CALL值无效！", e1);
					}
				}
				calls.offer(spt);
			}
			return calls;
		}

		private void parserFunction(Element e, SqlFactory sf)
				throws XmlParserException {
			String prefix = e.attributeValue(SqlParser.PREFIX);
			String name = e.attributeValue(SqlParser.NAME);
			String m = e.attributeValue(SqlParser.MASK);
			boolean mask = !(m != null && m.equals("false"));
			SqlVar[] asv = this.getVars(e, SqlParser.VAR, sf);
			SqlTable[] ast = this.getTables(e, SqlParser.TABLE, sf);
			SqlVar out = this.getVars(e, SqlParser.RETURN, sf)[0];
			String code = e.getTextTrim();
			if (code.equals(""))
				code = null;
			try {
				sf.createFunction(prefix, name, mask, asv, ast, out, code);
			} catch (SQLExistsException e1) {
				throw new XmlParserException("函数" + name + "已经存在！定义重复！", e1);
			}

		}

		private SqlDialect parserInfo(Element e, SqlInfo si)
				throws XmlParserException {
			String abc = e.element("sql").getTextTrim();
			if (!abc.equals(SqlParser.INFO_SQL_ALL)) {
				si.setAsSupportAllDatabases(false);
				Dialect d = null;
				for (Dialect dia : Dialect.values()) {
					if (dia.name().toLowerCase().equals(abc)) {
						d = dia;
						break;
					}
				}
				if (d == null) {
					throw new XmlParserException("Sql方言" + abc + "暂不支持！");
				} else if (d.getSqlDialect() != si.getJdbcUtils().getDialect()) {
					throw new XmlParserException("该文件流要求的数据库"
							+ d.getDatabaseProductname()
							+ "与数据源数据库"
							+ si.getJdbcUtils().getDialect()
									.getDatabaseProductName() + "不兼容！");
				}
				return d.getSqlDialect();
			} else {
				si.setAsSupportAllDatabases(true);
				return si.getJdbcUtils().getDialect();
			}
		}

		@SuppressWarnings("unchecked")
		private void parserInjector(Element e, SqlFactory sf)
				throws XmlParserException {
			SqlInjector si;
			try {
				si = new SqlInjector(Class.forName(e
						.attributeValue(SqlParser.CLASS)));
			} catch (ClassNotFoundException e1) {
				throw new XmlParserException("注入器解析失败！未找到类"
						+ e.attributeValue(SqlParser.CLASS), e1);
			}

			Iterator<Element> it = e.elementIterator(SqlParser.FUNCTION);
			while (it.hasNext()) {
				Element ie = it.next();
				try {
					si.bind(sf.getDatabase().getFunction(
							ie.attributeValue(SqlParser.NAME)),
							ie.attributeValue(SqlParser.INJECTOR_METHOD));
				} catch (SQLNotExistsException e1) {
					throw new XmlParserException("注入器解析失败！未找到函数"
							+ ie.attributeValue(SqlParser.NAME), e1);
				} catch (NoSuchMethodException e1) {
					throw new XmlParserException("注入器解析失败！未找到类中的setter方法"
							+ ie.attributeValue(SqlParser.NAME), e1);
				}
			}

			it = e.elementIterator(SqlParser.PROCEDURE);
			while (it.hasNext()) {
				Element ie = it.next();
				try {
					si.bind(sf.getDatabase().getProcedure(
							ie.attributeValue(SqlParser.NAME)),
							ie.attributeValue(SqlParser.INJECTOR_METHOD));
				} catch (SQLNotExistsException e1) {
					throw new XmlParserException("注入器解析失败！未找到存储过程"
							+ ie.attributeValue(SqlParser.NAME), e1);
				} catch (NoSuchMethodException e1) {
					throw new XmlParserException("注入器解析失败！未找到类中的setter方法"
							+ ie.attributeValue(SqlParser.NAME), e1);
				}
			}
			try {
				sf.getDatabase().addInjector(si);
			} catch (SQLExistsException e1) {
				throw new XmlParserException("注入器" + si.getSupportClass()
						+ "已经存在！", e1);
			}
		}

		private void parserMappings(Element e, SqlFactory sf)
				throws XmlParserException {
			@SuppressWarnings("unchecked")
			Iterator<Element> ie = e.elementIterator(SqlParser.MAPPING);
			while (ie.hasNext()) {
				Element it = ie.next();
				try {
					Class<?> cls = Class.forName(it
							.attributeValue(SqlParser.CLASS));
					SqlTable st = sf.getDatabase().getTable(
							it.attributeValue(SqlParser.TABLE));
					st.getJdbcUtils().getMappings()
							.register(cls, it.attributeValue(SqlParser.VALUE));
				} catch (ClassNotFoundException e1) {
					throw new XmlParserException("类型"
							+ it.attributeValue(SqlParser.CLASS) + "未找到", e1);
				} catch (SQLNotExistsException e1) {
					throw new XmlParserException("表格"
							+ it.attributeValue(SqlParser.TABLE) + "不存在", e1);
				} catch (MappingException e1) {
					throw new XmlParserException("类型"
							+ it.attributeValue(SqlParser.CLASS) + "不符合映射规则",
							e1);
				}
			}
		}

		private void parserPackUtils(Element e, SqlInfo si)
				throws XmlParserException {
			@SuppressWarnings("unchecked")
			Iterator<Element> ie = e.elementIterator(SqlParser.PROPERTY_EDITOR);
			while (ie.hasNext()) {
				Element it = ie.next();
				try {
					Class<?> cls = Class.forName(it
							.attributeValue(SqlParser.CLASS));
					Object o = cls.newInstance();
					if (o instanceof PropertyEditor) {
						si.getJdbcUtils().getMappings().getPackUtils()
								.register((PropertyEditor) o, cls);
					} else {
						throw new XmlParserException("类型"
								+ it.attributeValue(SqlParser.CLASS)
								+ "并未实现PropertyEditor接口");
					}
				} catch (ClassNotFoundException e1) {
					throw new XmlParserException("类型"
							+ it.attributeValue(SqlParser.CLASS) + "未找到", e1);
				} catch (InstantiationException e1) {
					throw new XmlParserException("造型失败", e1);
				} catch (IllegalAccessException e1) {
					throw new XmlParserException("造型失败", e1);
				}
			}
		}

		private void parserProcedure(Element e, SqlFactory sf)
				throws XmlParserException {
			String prefix = e.attributeValue(SqlParser.PREFIX);
			String name = e.attributeValue(SqlParser.NAME);
			String m = e.attributeValue(SqlParser.MASK);
			boolean mask = !(m != null && m.equals("false"));
			SqlVar[] asv = this.getVars(e, SqlParser.VAR, sf);
			SqlTable[] ast = this.getTables(e, SqlParser.TABLE, sf);
			SqlFunction[] asf = this.getFunctions(e, SqlParser.FUNCTION, sf);
			SqlVar[] out = this.getVars(e, SqlParser.RETURN, sf);
			String code = e.getTextTrim();
			if (code.equals(""))
				code = null;
			try {
				sf.createProcedure(prefix, name, mask, asv, ast, asf, code, out);
			} catch (SQLExistsException e1) {
				throw new XmlParserException("存储过程" + name + "已经存在！定义重复！", e1);
			}
		}

		private void parserTable(Element e, SqlFactory sf)
				throws XmlParserException {
			String name = e.attributeValue(SqlParser.NAME);
			String prefix = e.attributeValue(SqlParser.PREFIX);
			String ma = e.attributeValue(SqlParser.MASK);
			boolean mask = !(ma != null && ma.equals("false"));
			SqlTable st;
			try {
				st = sf.createTable(prefix, name, mask,
						this.getVars(e, SqlParser.VAR, sf));
			} catch (SQLExistsException e1) {
				throw new XmlParserException("表格" + name + "重复!", e1);
			}

			/**
			 * 外键。
			 */
			Matcher m = pv.matcher(get(e, SqlParser.FOREIGN, SqlParser.VAR));
			while (m.find()) {
				try {
					st.setForeignKey(m.group(1),
							sf.getDatabase().getTable(m.group(2)), m.group(3));
				} catch (SQLException e1) {
					throw new XmlParserException(st.getName() + "外键"
							+ m.group(1) + "解析失败!", e1);
				}
			}

			/**
			 * 非空字段。
			 */
			m = v.matcher(get(e, SqlParser.NOTNULL, SqlParser.VAR));
			while (m.find()) {
				try {
					String nm;
					try {
						nm = st.getVar(Integer.valueOf(m.group(1))).getName();
					} catch (NumberFormatException nfe) {
						nm = m.group(1);
					}
					st.setNotNull(nm);
				} catch (SQLNotExistsException e1) {
					throw new XmlParserException(
							"序号为" + m.group(1) + "的字段不存在！", e1);
				}
			}

			/**
			 * 值唯一字段。
			 */
			m = v.matcher(get(e, SqlParser.UNIQUE, SqlParser.VAR));
			while (m.find()) {
				try {
					String nm;
					try {
						nm = st.getVar(Integer.valueOf(m.group(1))).getName();
					} catch (NumberFormatException nfe) {
						nm = m.group(1);
					}
					st.setUniqueVar(nm);
				} catch (SQLNotExistsException e1) {
					throw new XmlParserException(
							"序号为" + m.group(1) + "的字段不存在！", e1);
				}
			}

			/**
			 * 主键。
			 */
			m = v.matcher(get(e, SqlParser.KEY, SqlParser.VAR));

			ManagerArray<String, String> mo = new ManagerArray<String, String>(
					null, String.class) {

				@Override
				protected String getName(String t) {
					return t;
				}

				@Override
				protected boolean isEqual(String v1, String v2) {
					return v1.equals(v2);
				}

			};

			while (m.find()) {
				try {
					String nm;
					try {
						nm = st.getVar(Integer.valueOf(m.group(1))).getName();
					} catch (NumberFormatException nfe) {
						nm = m.group(1);
					}
					try {
						mo.add(nm);
					} catch (SQLExistsException e1) {
						logger.error("已存在字段" + nm);
					}
				} catch (SQLNotExistsException e1) {
					throw new XmlParserException(
							"序号为" + m.group(1) + "的字段不存在！", e1);
				}
			}
			if (mo.getSize() > 0)
				try {
					st.setPrimaryKeys(mo.getAll());
					String x = e.element(SqlParser.KEY).attributeValue(
							SqlParser.AUTO_INCREMENT);
					if (x != null && x.equals("true")) {
						st.setAutoIncrement(true);
						if (!st.isAutoIncrement()) {
							logger.error("配置文件中的主键"
									+ Arrays.toString(st.getPrimaryKeys())
									+ "不符合设置为AutoIncrement的要求。");
						}
					}
				} catch (SQLNotExistsException e1) {
					throw new XmlParserException("主键解析失败！", e1);
				}

		}

		private void parserType(Element e, SqlFactory sf)
				throws XmlParserException {
			int id = Integer.valueOf(e.attributeValue(SqlParser.ID));
			SqlVarType name;
			try {
				name = SqlVarType.valueOf(e.attributeValue(SqlParser.NAME));
			} catch (IllegalArgumentException e1) {
				throw new XmlParserException("Sql不支持该类型"
						+ e.attributeValue(SqlParser.NAME), e1);
			}
			String length = e.attributeValue(SqlParser.LENGTH);
			if (length == null || length.equals(""))
				length = "0";
			int len = Integer.valueOf(length);
			String def = e.attributeValue(SqlParser.DEFAULT);
			if (def != null && def.equals(""))
				def = null;
			try {
				sf.createType(name, len, def, id);
			} catch (SQLException e1) {
			}
		}

		SqlFactory parserXml(Element root, SqlInfo si)
				throws XmlParserException {
			Map<String, Collection<Element>> temp = new LinkedHashMap<String, Collection<Element>>();
			temp.put(SqlParser.INFO, new LinkedHashSet<Element>());
			temp.put(SqlParser.TYPE, new LinkedHashSet<Element>());
			temp.put(SqlParser.TABLE, new LinkedHashSet<Element>());
			temp.put(SqlParser.FUNCTION, new LinkedHashSet<Element>());
			temp.put(SqlParser.PROCEDURE, new LinkedHashSet<Element>());
			temp.put(SqlParser.INJECTOR, new LinkedHashSet<Element>());
			temp.put(SqlParser.CALL, new LinkedHashSet<Element>());
			temp.put(SqlParser.MAPPINGS, new LinkedHashSet<Element>());
			temp.put(SqlParser.PACK_UTILS, new LinkedHashSet<Element>());

			/**
			 * 扫描一级子元素。
			 */
			@SuppressWarnings("rawtypes")
			Iterator ie = root.elementIterator();
			while (ie.hasNext()) {
				Element it = (Element) ie.next();
				Collection<Element> c = temp.get(it.getName());
				if (c != null)
					c.add(it);
			}
			/**
			 * 解析info参数。主要用于验证数据源和该xml文件是否兼容。
			 */
			if (temp.get(SqlParser.INFO).size() == 0) {
				throw new XmlParserException("未找到INFO信息。");
			} else
				for (Element e : temp.get(SqlParser.INFO)) {
					parserInfo(e, si);
					break;
				}

			for (Element e : temp.get(SqlParser.PACK_UTILS)) {
				parserPackUtils(e, si);
				break;
			}

			/**
			 * 创建SQL工厂。
			 */
			SqlFactory sf = new SqlFactory(si);

			/**
			 * 获取所有字段类型。
			 */
			for (Element e : temp.get(SqlParser.TYPE)) {
				parserType(e, sf);
			}

			/**
			 * 
			 * 解析表格。
			 */
			for (Element e : temp.get(SqlParser.TABLE)) {
				parserTable(e, sf);
			}

			for (Element e : temp.get(SqlParser.MAPPINGS)) {
				parserMappings(e, sf);
				break;
			}

			/**
			 * 解析函数。
			 */

			for (Element e : temp.get(SqlParser.FUNCTION)) {
				parserFunction(e, sf);
			}

			/**
			 * 解析存储过程。
			 */

			for (Element e : temp.get(SqlParser.PROCEDURE)) {
				parserProcedure(e, sf);
			}

			/**
			 * 解析注入器。
			 */

			for (Element e : temp.get(SqlParser.INJECTOR)) {
				parserInjector(e, sf);
			}

			/**
			 * 解析存储过程执行。
			 */
			for (Element e : temp.get(SqlParser.CALL)) {
				sf.getDatabase().setInitialData(parserCall(e, sf));
				break;
			}
			return sf;
		}
	}

	/**
	 * 数据库输出流对象。用于导出数据库对象。
	 * 
	 * @author Daniel
	 * @datetime 2012-11-2上午11:52:29
	 * 
	 */
	private static class XmlWriter {

		private Element root;

		private Document doc;

		private ManagerTypes motp = new ManagerTypes();

		private ManagerObjects<SqlTable> mot = new ManagerObjects<SqlTable>(
				SqlTable.class);
		private ManagerObjects<SqlFunction> mof = new ManagerObjects<SqlFunction>(
				SqlFunction.class);
		private ManagerObjects<SqlProcedure> mop = new ManagerObjects<SqlProcedure>(
				SqlProcedure.class);
		private final FormatSql beauty;
		private final FormatSql indent;

		XmlWriter(FormatSql beauty) {
			this.beauty = beauty;
			this.indent = new IndentFormat();
		}

		private void add(Element e) {
			root.addText("\n	");
			root.add(e);
		}

		private <T extends SqlObject> void addAll(ManagerObjects<T> mo, T[] ts) {
			for (T t : ts) {
				initMO(t);
				try {
					mo.add(t);
				} catch (SQLExistsException e) {
				}
			}
		}

		private void addCallElement(Queue<ProcedureToken> id) {
			if (id == null || id.size() == 0) {
				return;
			}
			String value = "";
			for (ProcedureToken spt : id) {
				value += "\n		#" + spt.getSqlProcedure().getName() + "{";
				for (Object[] svt : spt.getTokens()) {
					value += "\n			";
					for (int i = 0; i < svt.length - 1; i++) {
						value += "'" + svt[i] + "',";
					}
					value += "'" + svt[svt.length - 1] + "';";
				}
				value += "\n		}\n";
			}
			root.addText("\n	");
			Element c = root.addElement(SqlParser.CALL);
			c.addText(value + "	");
		}

		private void addFunctionElement(SqlFunction sf, boolean number2name)
				throws SqlFormatException {
			Element e = this.getQElement(SqlParser.FUNCTION);

			addhead(sf, e);

			idAttr(e, SqlParser.VAR, sf.getVars());

			attr(e, SqlParser.TABLE, sf.getTables());

			idAttr(e, SqlParser.RETURN, sf.getOutVar());
			/**
			 * 这一步把数值转换成字符。
			 */
			e.addText(formatSql(sf, sf.getSql(), number2name));

			add(e);
		}

		/**
		 * 添加 NAME属性，PREFIX属性以及MASK属性。
		 * 
		 * @param t
		 * @param e
		 */
		private <T extends AbstractSql> void addhead(T t, Element e) {
			String prefix = t.getPrefix();
			/**
			 * 规则。
			 */
			if (prefix != null && !prefix.equals(t.getSqlInfo().getPrefix())) {
				e.addAttribute(SqlParser.PREFIX, t.getPrefix());
			}
			e.addAttribute(SqlParser.NAME, t.getName());

			if (!t.getMask())
				e.addAttribute(SqlParser.MASK, "false");
		}

		private void addInjectorElement(SqlInjector si) {
			Element e = this.getQElement(SqlParser.INJECTOR);
			e.addAttribute(SqlParser.CLASS, si.getSupportClass().getName());
			Map<String, SqlProcedure> pm = si.getProcedureBindings();
			for (String key : pm.keySet()) {
				e.addElement(SqlParser.PROCEDURE)
						.addAttribute(SqlParser.NAME, pm.get(key).getName())
						.addAttribute(SqlParser.INJECTOR_METHOD, key);
			}
			Map<String, SqlFunction> fm = si.getFunctionBindings();
			for (String key : fm.keySet()) {
				e.addElement(SqlParser.FUNCTION)
						.addAttribute(SqlParser.NAME, fm.get(key).getName())
						.addAttribute(SqlParser.INJECTOR_METHOD, key);
			}
			add(e);
		}

		private void addMappingsElement(
				Map<SqlTable, Collection<Mapping<?>>> map) {
			if (map.size() == 0)
				return;
			Element e = this.getQElement(SqlParser.MAPPINGS);
			for (SqlTable t : map.keySet()) {
				for (Mapping<?> m : map.get(t)) {
					Element ie = e.addElement(SqlParser.MAPPING);
					ie.addAttribute(SqlParser.CLASS, m.getDefinedClass()
							.getName());
					ie.addAttribute(SqlParser.TABLE, t.getName());
					ie.addAttribute(SqlParser.VALUE, m.toString());
				}
			}
			add(e);
		}

		private void addPackUtilsElement(PackUtils pu) {
			Collection<Class<?>> xx = pu.values();
			if (xx.size() > 0) {
				Element e = this.getQElement(SqlParser.PACK_UTILS);
				for (Class<?> cls : xx) {
					e.addElement(SqlParser.PROPERTY_EDITOR).addAttribute(
							SqlParser.CLASS, cls.getName());
				}
				add(e);
			}
		}

		private void addProcedureElement(SqlProcedure sp, boolean number2name)
				throws SqlFormatException {
			Element e = this.getQElement(SqlParser.PROCEDURE);

			addhead(sp, e);

			idAttr(e, SqlParser.VAR, sp.getVars());

			attr(e, SqlParser.TABLE, sp.getTables());

			attr(e, SqlParser.FUNCTION, sp.getFunctions());

			/**
			 * 这一步把数值转换成字符。
			 */

			e.addText(formatSql(sp, sp.getSql(), number2name));
			add(e);
		}

		private void addTableElement(SqlTable st) {
			Element troot = this.getQElement(SqlParser.TABLE);

			addhead(st, troot);

			attr(troot, SqlParser.KEY, SqlParser.VAR, st.getPrimaryKeys());
			if (st.isAutoIncrement() && troot.element(SqlParser.KEY) != null) {
				troot.element(SqlParser.KEY).addAttribute(
						SqlParser.AUTO_INCREMENT, "true");
			}

			attr(troot, SqlParser.UNIQUE, SqlParser.VAR, st.getUniqueVars());

			/**
			 * 获取有效的
			 */
			ManagerObjects<SqlVar> mo = new ManagerObjects<SqlVar>(SqlVar.class);
			for (SqlVar v : st.getNotNullVars()) {
				if (!st.isUnique(v.getName()) && !st.isForeignKey(v.getName())
						&& !st.isKey(v.getName()))
					try {
						mo.add(v);
					} catch (SQLExistsException e) {
					}
			}

			attr(troot, SqlParser.NOTNULL, SqlParser.VAR, mo.getAll());

			idAttr(troot, SqlParser.VAR, st.getVars());

			fkeyAttr(troot, SqlParser.FOREIGN, SqlParser.VAR, st);

			add(troot);
		}

		private void addTypeElement(SqlType st) {
			Element vroot = this.getQElement(SqlParser.TYPE);
			vroot.addAttribute(SqlParser.ID, String.valueOf(st.getIndex()));
			vroot.addAttribute(SqlParser.NAME, st.getJdbcName());
			if (st.hasLength())
				vroot.addAttribute(SqlParser.LENGTH,
						String.valueOf(st.getLength()));
			vroot.addAttribute(SqlParser.DEFAULT, st.getDefaultValue());
			add(vroot);
		}

		protected <T extends SqlObject> void attr(Element e, String sub,
				String attrName, T... st) {
			Element se = getQElement(sub);
			if (attr(se, attrName, st))
				e.add(se);
		}

		protected <T extends SqlObject> boolean attr(Element e,
				String attrName, T... st) {
			String attr = "";
			for (T t : st) {
				attr += t.getName() + " ";
			}
			if (!attr.equals("")) {
				e.addAttribute(attrName, attr.trim());
			}
			return !attr.equals("");
		}

		protected void fkeyAttr(Element e, String sub, String attrName,
				SqlTable st) {
			String attr = "";
			for (SqlVar v : st.getForeignKeys()) {
				attr += v.getName() + "="
						+ st.getForeignTable(v.getName()).getName() + "."
						+ st.getForeignKey(v.getName()).getName() + " ";
			}
			if (!attr.equals(""))
				e.addElement(sub).addAttribute(attrName, attr.trim());
		}

		private String formatSql(Object obj, String sql, boolean number2name)
				throws SqlFormatException {
			if (number2name)
				sql = new NumberToNameFormat(obj).format(sql);
			else
				sql = new NameToNumberFormat(obj).format(sql);
			return "\n" + indent.format(beauty.format(sql)) + "\n	";
		}

		private Element getQElement(String local) {
			return DocumentHelper.createElement(DocumentHelper.createQName(
					local, root.getNamespace()));
		}

		protected <T extends SqlObject> void idAttr(Element e, String attrName,
				T... st) {
			String attr = "";
			for (T t : st) {
				attr += t.getId() + " ";
			}
			if (!attr.equals(""))
				e.addAttribute(attrName, attr.trim());
		}

		private void init(SqlDatabase sl) {
			DocumentFactory df = DocumentFactory.getInstance();
			root = df.createElement(SqlParser.ROOT,
					"http://www.icymint.me/xml/sage/sql");
			Namespace ns = df.createNamespace("xsi",
					"http://www.w3.org/2001/XMLSchema-instance");
			root.addAttribute(
					df.createQName("schemaLocation", ns),
					"http://www.icymint.me/xml/sage/sql http://www.icymint.me/xml/sage/sql/sage.xsd");
			root.addElement(SqlParser.INFO)
					.addElement(SqlParser.INFO_SQL)
					.addText(
							sl.getSqlInfo().isSupportAllDatabases() ? SqlParser.INFO_SQL_ALL
									: sl.getJdbcUtils().getDialect()
											.getDialectName());
			doc = df.createDocument(root);

			motp.clear();
			motp.addAll(sl.getTypes());
			mot.clear();
			mot.addAll(sl.getTables());
			mof.clear();
			mof.addAll(sl.getFunctions());
			mop.clear();
			mop.addAll(sl.getProcedures());
			initMO(sl);
		}

		private <T extends SqlObject> void initMO(T obj) {
			if (obj instanceof SqlDatabase) {
				for (SqlType t : ((SqlDatabase) obj).getTypes()) {
					try {
						motp.add(t);
					} catch (SQLExistsException e) {
					}
				}
			}
			if (obj instanceof VarManager) {
				for (SqlVar t : ((VarManager) obj).getVars()) {
					try {
						motp.add(t.getType());
					} catch (SQLExistsException e) {
					}
				}
			}
			if (obj instanceof TableManager) {
				addAll(mot, ((TableManager) obj).getTables());
			}
			if (obj instanceof FunctionManager) {
				addAll(mof, ((FunctionManager) obj).getFunctions());
			}
			if (obj instanceof ProcedureManager) {
				addAll(mop, ((ProcedureManager) obj).getProcedures());
			}
		}

		void save(OutputStream os, SqlDatabase sl, boolean number2name)
				throws IOException, SqlFormatException {
			init(sl);

			for (SqlType t : motp.getAll()) {
				this.addTypeElement(t);
			}

			Map<SqlTable, Collection<Mapping<?>>> map = new LinkedHashMap<SqlTable, Collection<Mapping<?>>>();
			for (SqlTable t : mot.getAll()) {
				this.addTableElement(t);
				Collection<Mapping<?>> xx = t.getJdbcUtils().getMappings()
						.getAll();
				if (xx.size() > 0) {
					map.put(t, xx);
				}
			}

			for (SqlFunction f : mof.getAll()) {
				this.addFunctionElement(f, number2name);
			}

			for (SqlProcedure p : mop.getAll()) {
				this.addProcedureElement(p, number2name);
			}

			for (SqlInjector si : sl.getInjectors()) {
				addInjectorElement(si);
			}

			addCallElement(sl.getInitialData());

			addMappingsElement(map);
			addPackUtilsElement(sl.getJdbcUtils().getMappings().getPackUtils());

			/**
			 * 保存。
			 */
			XMLWriter x;
			try {
				OutputFormat of = OutputFormat.createPrettyPrint();
				of.setEncoding("UTF-8");
				of.setTrimText(false);
				of.setIndent("	");
				x = new XMLWriter(os, of);
				x.write(doc);
			} catch (UnsupportedEncodingException e1) {
			}
		}

	}

	protected DefaultSqlParser() {
		super(DefaultSqlParser.class.getResource("sage.xsd"));
	}

	@Override
	protected SqlFactory parserXml(Element root, SqlInfo si)
			throws XmlParserException {
		return new XmlReader().parserXml(root, si);
	}

	@Override
	public void storeXml(OutputStream xml, SqlFactory sf)
			throws XmlStoreException {
		try {
			new XmlWriter(sf.getDatabase().getDialect().getFormat()).save(xml,
					sf.getDatabase(), sf.isReadable());
		} catch (IOException e) {
			throw new XmlStoreException("保存失败！", e);
		} catch (SqlFormatException e) {
			throw new XmlStoreException("保存失败！", e);
		}
	}

}
